/*====================================================================
		rstring.c

		rFunc InterBase UDF library.
		String functions.

		Copyright 1998-2001 Polaris Software
		http://polesoft.da.ru
		mailto: rFunc@mail.ru

		Version 2.0.1.2
====================================================================== */

#include <ibase.h>
#include <string.h>
#include "rfunc.h"
#include "rstring.h"
#include "win1251.h"

char *delims = " "; // empty delimiters

unsigned char* EXPORT fn_chr(ARG(short*, i))
ARGLIST(short *i)
{
	unsigned char *s = (unsigned char*) MALLOC (2);
	s[0] = (unsigned char) *i;
	s[1] = '\0';
	return s;
}

short EXPORT fn_ord(ARG(unsigned char*, s))
ARGLIST(unsigned char *s)
{
	return (short) s[0];
}

char* EXPORT fn_ltrim(ARG(char*, s))
ARGLIST(char *s)
{
	char *c;
	c = s;
	while (*c && strchr(delims, *c)) c++;
	return c;
}

char* EXPORT fn_rtrim(ARG(char*, s))
ARGLIST(char *s)
{
	long n;
	n = strlen(s);
	if (n) {
		n--;
		while (strchr(delims, s[n])) n--;
		s[n+1] = '\0';
	}
	return s;
}

char* EXPORT fn_trim(ARG(char*, s))
ARGLIST(char *s)
{
	return fn_rtrim(fn_ltrim(s));
}

unsigned char* EXPORT fn_rupper(ARG(unsigned char*, s))
ARGLIST(unsigned char *s)
{
	unsigned char *ptr = s;

	while (*ptr) *ptr = upper_win1251[(short)*ptr++];
	return s;
}

unsigned char* EXPORT fn_rlower(ARG(unsigned char*, s))
ARGLIST(unsigned char *s)
{
	unsigned char *ptr = s;

	while (*ptr) *ptr = lower_win1251[(short)*ptr++];
	return s;
}

unsigned char* EXPORT fn_rlatin(ARG(unsigned char*, s))
ARGLIST(unsigned char *s)
{
	unsigned char *ptr = s;

	while (*ptr) *ptr = latin_win1251[(short)*ptr++];
	return s;
}

unsigned char* rntranslit(ARG(unsigned char*, s), ARG(long, maxlength))
ARGLIST(unsigned char *s)
ARGLIST(long maxlength)
{
	unsigned char *buffer = (unsigned char*) MALLOC (maxlength);
	unsigned char *sptr = s;
	int i, l = 0;
	short r;

	while (*sptr)
	{
		 if (r = translit_win1251[(short)*sptr])
		 {
			i = 0;
			 while ((buffer[l++] = TYP_win1251[r-1][i++]) && (l < maxlength));
			 l--;
		 } else
			 buffer[l++] = *sptr;
		 sptr++;
		if (l >= maxlength) break;
	}
	buffer[l] = '\0';
	return buffer;
}

unsigned char* EXPORT fn_rtranslit(ARG(unsigned char*, s))
ARGLIST(unsigned char *s)
{
	return rntranslit(s, shortlen);
}

unsigned char* EXPORT fn_longrtranslit(ARG(unsigned char*, s))
ARGLIST(unsigned char *s)
{
	return rntranslit(s, longlen);
}

char* EXPORT fn_substr(ARG(char*, s), ARG(long*, m), ARG(long*, n))
ARGLIST(char *s)
ARGLIST(long *m)
ARGLIST(long *n)
{
	long left, right;
	long len = strlen(s);

	left = (*m < 0) ? len + *m + 1 : *m;
	right = (*n < 0) ? left - 1 : left - 1 + *n - 1;
	left = (*n < 0) ? left + *n : left - 1;

	left = (left < 0) ? 0 : left;
	right = (right > len - 1) ? len - 1 : right;

	if (!*n || (left > len - 1) || (right < 0))
		return "";

	s[right + 1] = '\0';
	return &s[left];
}

char* strnrepeat(ARG(char*, s), ARG(long*, c), ARG(long, maxlength))
ARGLIST(char *s)
ARGLIST(long *c)
ARGLIST(long maxlength)
{
	long i = 0;
	long j = 0;
	long l = strlen(s);
	long n=(*c*l<maxlength) ? *c*l : maxlength-1;
	char *buffer = (char*) MALLOC (maxlength);
	if (l < 0) return "";
	while (i < n)
	{
		buffer[i++] = s[j++];
		if (!s[j]) j = 0;
	}
	buffer[i] = '\0';
	return buffer;
}

char* EXPORT fn_strrepeat(ARG(char*, s), ARG(long*, c))
ARGLIST(char *s)
ARGLIST(long *c)
{
	return strnrepeat(s, c, shortlen);
}

char* EXPORT fn_longstrrepeat(ARG(char*, s), ARG(long*, c))
ARGLIST(char *s)
ARGLIST(long *c)
{
	return strnrepeat(s, c, longlen);
}

char* strnstuff(ARG(char*, s), ARG(long*, spos), ARG(long*, dcount), ARG(char*, is), ARG(long, maxlength))
ARGLIST(char *s)
ARGLIST(long *spos)
ARGLIST(long *dcount)
ARGLIST(char *is)
ARGLIST(long maxlength)
{
	long i;
	long j = 0;
	long l = 0;
	long n = strlen(s);
	char *buffer = (char*) MALLOC (maxlength);
	n = (*spos <= n)? ((*spos > 0)? *spos - 1 : 0) : n;
	while (l<n)
		buffer[l++] = s[j++];
	for (i=0; (is[i] && (l<maxlength)); i++)
		buffer[l++] = is[i];
	i = j + *dcount;
	while (s[j] && (l < maxlength))
	{
		if(j >= i) buffer[l++] = s[j];
		j++;
	}
	buffer[l] = '\0';
	return buffer;
}

char* EXPORT fn_strstuff(ARG(char*, s), ARG(long*, spos), ARG(long*, dcount), ARG(char*, is))
ARGLIST(char *s)
ARGLIST(long *spos)
ARGLIST(long *dcount)
ARGLIST(char *is)
{
	return strnstuff(s, spos, dcount, is, shortlen);
}

char* EXPORT fn_longstrstuff(ARG(char*, s), ARG(long*, spos), ARG(long*, dcount), ARG(char*, is))
ARGLIST(char *s)
ARGLIST(long *spos)
ARGLIST(long *dcount)
ARGLIST(char *is)
{
	return strnstuff(s, spos, dcount, is, longlen);
}

char* strnreplace(ARG(char*, s), ARG(char*, froms), ARG(char*, tos), ARG(long, maxlength))
ARGLIST(char *s)
ARGLIST(char *froms)
ARGLIST(char *tos)
ARGLIST(long maxlength)
{
	long sn = strlen(froms);
	char *buffer = (char*) MALLOC (maxlength);
	char *sptr = s, *ptr = s, *bptr = buffer;

	while (ptr = strstr(ptr, froms))
	{
		while (sptr != ptr && (bptr-buffer<maxlength-1)) *bptr++ = *sptr++;
		sptr = tos;
		while (*sptr && (bptr-buffer<maxlength-1)) *bptr++ = *sptr++;
		ptr = ptr + sn;
		sptr = ptr;
	}
	while (*sptr && (bptr-buffer<maxlength-1))
		*bptr++ = *sptr++;
	*bptr = '\0';
	return buffer;
}

char* EXPORT fn_strreplace(ARG(char*, s), ARG(char*, froms), ARG(char*, tos))
ARGLIST(char *s)
ARGLIST(char *froms)
ARGLIST(char *tos)
{
	return strnreplace(s, froms, tos, shortlen);
}

char* EXPORT fn_longstrreplace(ARG(char*, s), ARG(char*, froms), ARG(char*, tos))
ARGLIST(char *s)
ARGLIST(char *froms)
ARGLIST(char *tos)
{
	return strnreplace(s, froms, tos, longlen);
}

long EXPORT fn_strpos(ARG(char*, str1), ARG(char*, str2))
ARGLIST(char *str1)
ARGLIST(char *str2)
{
	char *ptr;
	if (!*str1) return 0;
	ptr = strstr(str2, str1);
	return (ptr) ? (ptr-str2+1) : 0;
}

long EXPORT fn_strlen(ARG(char*, s))
ARGLIST(char *s)
{
	return strlen(s);
}

long EXPORT fn_strcount(ARG(char*, str1), ARG(char*, str2))
ARGLIST(char *str1)
ARGLIST(char *str2)
{
	char *ptr;
	long  r = 0;
	long  len = strlen(str1);

	if (!len || !*str2) return 0;
	ptr = str2;
	while (ptr = strstr(ptr, str1))
	{
		ptr += len;
		r++;
	}
	return r;
}

long EXPORT fn_wordcount(ARG(char*, s), ARG(char*, delim), ARG(long*, flag))
ARGLIST(char *s)
ARGLIST(char *delim)
ARGLIST(long *flag)
{
	char *c = s;
	long r = 0;
	long d = 0;

	if (!*c) return r;
	while (*c)
		if (strchr(delim, *c++))
			d++; /* counting words */
		else {
			while (*c && !strchr(delim, *c)) c++;
			r++; /* counting full words */
		}
	if (*flag) {
		if (!strchr(delim, *--c)) d++;
		return d;
	} else
		return r;
}

char* EXPORT fn_wordnum(ARG(char*, s), ARG(long*, n), ARG(char*, delim), ARG(long*, flag))
ARGLIST(char *s)
ARGLIST(long *n)
ARGLIST(char *delim)
ARGLIST(long *flag)
{
	char *c = s, *ptr;
	long r = 1;

	if (!*c) return "";
	while (*c && r!=*n)
		if (strchr(delim, *c++)) {
			if (*flag)
				r++;
			else
				while (*c && strchr(delim, *c)) c++;
		} else {
			while (*c && !strchr(delim, *c)) c++;
			if (!*flag) r++;
		}
	if (!*flag)
		while (*c && strchr(delim, *c)) c++;
	ptr = c;
	while (*ptr && !strchr(delim, *ptr)) ptr++;
	*ptr = '\0';

	return c;
}

char*	padnright(ARG(char*, s), ARG(short*, n), ARG(char*, c), ARG(long, maxlength))
ARGLIST(char *s)
ARGLIST(short *n)
ARGLIST(char *c)
ARGLIST(long maxlength)
{
	long i = 0;
	char *buffer = (char*) MALLOC (maxlength);
	char *ptr = s;

	while (*ptr && (i < maxlength - 1)) buffer[i++] = *ptr++;
	while ((i < *n) && (i < maxlength - 1)) buffer[i++] = *c;
	buffer[i] = '\0';
	return buffer;
}

char*	EXPORT fn_padright(ARG(char*, s), ARG(short*, n), ARG(char*, c))
ARGLIST(char *s)
ARGLIST(short *n)
ARGLIST(char *c)
{ return padnright(s, n, c, shortlen); }

char*	EXPORT fn_longpadright(ARG(char*, s), ARG(short*, n), ARG(char*, c))
ARGLIST(char *s)
ARGLIST(short *n)
ARGLIST(char *c)
{ return padnright(s, n, c, longlen); }

char*	padnleft(ARG(char*, s), ARG(short*, n), ARG(char*, c), ARG(long, maxlength))
ARGLIST(char *s)
ARGLIST(short *n)
ARGLIST(char *c)
ARGLIST(long maxlength)
{
	long l = strlen(s), i = 0;
	char *buffer = (char*) MALLOC (maxlength);
	char *ptr = s;

	while ((i < *n - l) && (i < maxlength - 1)) buffer[i++] = *c;
	while (*ptr && (i < maxlength - 1)) buffer[i++] = *ptr++;
	buffer[i] = '\0';
	return buffer;
}

char*	EXPORT fn_padleft(ARG(char*, s), ARG(short*, n), ARG(char*, c))
ARGLIST(char *s)
ARGLIST(short *n)
ARGLIST(char *c)
{ return padnleft(s, n, c, shortlen); }

char*	EXPORT fn_longpadleft(ARG(char*, s), ARG(short*, n), ARG(char*, c))
ARGLIST(char *s)
ARGLIST(short *n)
ARGLIST(char *c)
{ return padnleft(s, n, c, longlen); }
